@@ -30,11 +30,12 @@
 #include "action.h"
 #include "hashtab.h"
 #include "convert.h"
-#include "out_file_acl.h"
+#include "out_file_rule.h"
 #include "initial_policy.h"
 #include <seedit/common.h>
 #include <seedit/parse.h>
 #include "security_class.h"
+#include "out_macro.h"
 
 static void modify_rules();
 static void make_dir_list();
@@ -84,6 +85,24 @@ FILE *openfile(char *outdir, char *filename){
   return fp;
 }
 
+/*Declare macros useful for size optimization*/
+void declare_optimize_macro(FILE *policy_fp) {
+	fprintf(policy_fp, "define(`allow_file_wrs',`\n" \
+		"allow_file_w($1,$2)\n" \
+		"allow_file_r($1,$2)\n" \
+		"allow_file_s($1,$2)\n')\n");
+	fprintf(policy_fp, "define(`allow_file_rs',`\n"	\
+		"allow_file_r($1,$2)\n"	\
+		"allow_file_s($1,$2)\n')\n");
+	fprintf(policy_fp, "define(`allow_file_dev_wrs',`\n" \
+		"allow_file_dev_w($1,$2)\n" \
+		"allow_file_dev_r($1,$2)\n" \
+		"allow_file_dev_s($1,$2)\n')\n");
+	fprintf(policy_fp, "define(`allow_file_dev_rs',`\n" \
+		"allow_file_dev_r($1,$2)\n" \
+		"allow_file_dev_s($1,$2)\n')\n");
+
+}
 
 /**
  *  @name:	convert
@@ -92,76 +111,80 @@ FILE *openfile(char *outdir, char *filename){
  *  @args:	file_context_fp (FILE *) -> :FILE pointer to print file_contexts.
  *  @return:	none
  */
-void convert(char *outdir){
-  FILE *policy_fp=stdout;
-  FILE *file_context_fp=stdout;
-  FILE *unconfined_fp=stdout;
-  FILE *confined_fp = stdout;
-  FILE *customizable_types_fp=stdout;
-  FILE *homedir_template_fp=stdout;
-  FILE *userhelper_context_fp=stdout;
+void convert(char *outdir) {
+	FILE *policy_fp = stdout;
+	FILE *file_context_fp = stdout;
+	FILE *unconfined_fp = stdout;
+	FILE *confined_fp = stdout;
+	FILE *customizable_types_fp = stdout;
+	FILE *homedir_template_fp = stdout;
+	FILE *userhelper_context_fp = stdout;
+	FILE *profile_fp = stdout;
+       
+	if(outdir != NULL) {
+		policy_fp= openfile(outdir,"generated.conf");
+		file_context_fp = openfile(outdir,"file_contexts.m4");
+		unconfined_fp = openfile(outdir,"unconfined_domains");
+		confined_fp = openfile(outdir,"confined_domains");
+		customizable_types_fp=openfile(outdir,"customizable_types");
+		userhelper_context_fp=openfile(outdir,"userhelper_context.m4");
+		if(gProfile)
+			profile_fp = openfile(outdir, "profile.data");
+      	}  
 
-  if(outdir!=NULL){
-    policy_fp= openfile(outdir,"generated.conf");
-    file_context_fp = openfile(outdir,"file_contexts.m4");
-    unconfined_fp = openfile(outdir,"unconfined_domains");
-    confined_fp = openfile(outdir,"confined_domains");
-    customizable_types_fp=openfile(outdir,"customizable_types");
-    userhelper_context_fp=openfile(outdir,"userhelper_context.m4");
-    
-  }  
+	/* print default files */
+	include_file(get_base_policy_files()->security_class, policy_fp);
+	include_file(get_base_policy_files()->initial_sids, policy_fp);
+	include_file(get_base_policy_files()->access_vectors, policy_fp);
+	include_file(get_base_policy_files()->mcs, policy_fp);
+  
+	if(gDiet_by_attr || gProfile)
+		declare_optimize_macro(policy_fp);
 
- /* print default files */
+	/* print default all attribute */
+	declare_attributes(policy_fp);
+	/* print initial types */
+	declare_initial_types(policy_fp);
+	/* print "allow" at least necessary to configuration */
+	default_allow(policy_fp);
+	/**/
+	if(!gDir_search) {
+		fprintf(policy_fp,"###Dir search is not supported.\n");
+		fprintf(policy_fp,"allow domain file_type:dir { search };\n");
+	}
+	
+	/*used to sort file_context*/
+	TMP_fp = tmpfile();
+	if (TMP_fp == NULL) {
+		fprintf(stderr,"Error opening tmpfile\n");
+		exit(1);
+	}
 
-  include_file(get_base_policy_files()->security_class, policy_fp);
-  include_file(get_base_policy_files()->initial_sids, policy_fp);
-  include_file(get_base_policy_files()->access_vectors, policy_fp);
-  include_file(get_base_policy_files()->mcs, policy_fp);
+	if (domain_hash_table == NULL){
+		fprintf(stderr, "bug? domain table is not initialized.\n");
+		exit(1);
+	}
 
-  /* print default all attribute */
-  declare_attributes(policy_fp);
-  /* print initial types */
-  declare_initial_types(policy_fp);
-  /* print "allow" at least necessary to configuration */
-  /* print "socket allow"				     */
-  default_allow(policy_fp);
+	/* calculate relationship between file and label. */
+	create_file_label_table(domain_hash_table);	
 
-
-  /*used to sort file_context*/
-  TMP_fp = tmpfile();
-  if (TMP_fp == NULL){
-    fprintf(stderr,"Error opening tmpfile\n");
-    exit(1);
-  }
-
-  if (domain_hash_table == NULL){
-    fprintf(stderr, "bug? domain table is not initialized.\n");
-    exit(1);
-  }
-
-  /* calculate relationship between file and label. */
-  create_file_label_table(domain_hash_table);	
-#ifdef DIRSEARCH
-	create_dir_label_table(); 
-	//	print_dir_label_tab();
-#endif
-
-
-	//	print_file_label_tab();
+	if(gDir_search) {
+		create_dir_label_table(); 
+	}
 
 	modify_rules();
 
-#ifdef DIRSEARCH
-	make_dir_list();
-#endif
+	if(gDir_search) {
+		make_dir_list();
+	}
 
 	/* print "type .." of label on files */
 	out_file_type(policy_fp);
 	out_net_type(policy_fp);
-
+	
 	/* print "allow ..." */
 	out_allow(policy_fp,unconfined_fp,confined_fp);
-
+	
 	/* print "domain_auto_trans or domain_trans " */
 	out_domain_trans(policy_fp);
 
@@ -189,23 +212,24 @@ void convert(char *outdir){
 
 	out_userhelper_context(userhelper_context_fp);
 
-#ifdef DEBUG
-	//print_user();
-#endif
+	print_profile(profile_fp);
+	
 	if(policy_fp!=NULL)
-	  fclose(policy_fp);
+		fclose(policy_fp);
 	if(file_context_fp != NULL)
-	  fclose(file_context_fp);
+		fclose(file_context_fp);
 	if(unconfined_fp!=NULL)
-	  fclose(unconfined_fp);
+		fclose(unconfined_fp);
 	if(confined_fp!=NULL)
-	  fclose(confined_fp);
+		fclose(confined_fp);
 	if(customizable_types_fp!=NULL)
-	  fclose(customizable_types_fp);
+		fclose(customizable_types_fp);
 	if(homedir_template_fp!= NULL)
-	  fclose(homedir_template_fp);
+		fclose(homedir_template_fp);
 	if(userhelper_context_fp!=NULL)
-	  fclose(userhelper_context_fp);
+		fclose(userhelper_context_fp);
+	if(profile_fp!=NULL && profile_fp != stdout)
+		fclose(profile_fp);
 
 }
 
@@ -1265,15 +1289,17 @@ void out_fs_trans_acl(FILE *fp, DOMAIN *domain){
 void out_adm_other_acl(FILE *fp, DOMAIN *domain){
   int i;
   ADMIN_OTHER_RULE rule;
-  
+  char *macro;
   fprintf(fp, "##allowpriv rule\n");
   for(i = 0; i<domain->admin_rule_array_num; i++){
     rule = domain->admin_rule_array[i];
     if(rule.deny_flag==1){
       continue;
     }
-    fprintf(fp, "allow_admin_%s(%s)\n", rule.rule, domain->name);
     
+    macro = joint_str("allow_admin_",rule.rule);
+    out_optimized_macro(fp, macro, domain->name, NULL);
+    free(macro);
   }
   fprintf(fp, "##end of allowpriv rule\n");
   /*allowpriv part_relabel is in out_file_acl.c*/
@@ -1334,9 +1360,9 @@ out_acls(FILE *outfp, DOMAIN *domain)
 {
 
         out_file_acl(outfp, domain);
-#ifdef DIRSEARCH
-	out_dir_search(outfp,domain);
-#endif
+	if(gDir_search) {
+		out_dir_search(outfp,domain);
+	}
 	out_tmp_all(outfp,domain);
 	out_net_socket_acl(outfp,domain);
 	out_net_netif_acl(outfp,domain);
@@ -1569,7 +1595,9 @@ out_allow(FILE *outfp,FILE *unconfined_fp,FILE *confined_fp)
 
 		if(domain->program_flag){
 		  prefix = get_prefix(domain->name);
-		  fprintf(outfp,"bool %s_disable_trans false;\n",prefix);
+		  if(!gNoBool){
+			  fprintf(outfp,"bool %s_disable_trans false;\n",prefix);
+		  }
 		  free(prefix);
 		}
 		
@@ -1662,7 +1690,7 @@ int user_roles_other_role(char *user, char *role){
 
 
 /*if rule is changed, changed rule is returned*/
-FILE_ACL_RULE append_homedir_rule_to_domain(DOMAIN *domain, FILE_ACL_RULE rule, char **home_prefix_list){
+FILE_RULE append_homedir_rule_to_domain(DOMAIN *domain, FILE_RULE rule, char **home_prefix_list){
   
   char *path;
   int i;
@@ -1739,8 +1767,8 @@ void append_homedir_rule(){
   int i,j;
   DOMAIN *domain;
   HASH_NODE **domain_array;
-  FILE_ACL_RULE rule;
-  FILE_ACL_RULE rule_after;
+  FILE_RULE rule;
+  FILE_RULE rule_after;
   int rule_num_orig;
 
   homedir_list = converter_conf.homedir_list;
@@ -1794,68 +1822,67 @@ void modify_rules(){
 
 }
 
-void register_dirs(DOMAIN *domain, char **dir_list){
-  HASH_TABLE *dirshash;
-  int i;
-  int s;
-  int r;
-  struct stat buf;
-  dirshash = domain->dir_list;  
-  if (dirshash ==NULL){
-    dirshash = create_hash_table(FILE_ACL_TABLE_SIZE);
-    if (dirshash == NULL){
-      yyerror("memory shortage\n");
-      exit(1);
-    }
-  }
-  domain -> dir_list = dirshash;
-  for(i=0;dir_list[i]!=NULL;i++){
-    memset(&buf,0,sizeof(buf));
-    r=stat(dir_list[i],&buf);
-    if(r==0 && S_ISDIR(buf.st_mode)){
-      s = insert_element(dirshash, "1", dir_list[i]);  
-      if(s==-2){
-	;
-      }else if(s<0){
-	fprintf(stderr, "system error in line %d\n", __LINE__);
-	exit(1);
-      }
-    }
-    if(strcmp(dir_list[i],"~/")==0){
-      s = insert_element(dirshash, "1", dir_list[i]);  
-    }
+void register_dirs(DOMAIN *domain, char **dir_list) {
+	HASH_TABLE *dirshash;
+	int i;
+	int s;
 
-  }  
+	dirshash = domain->dir_list;  
+	if (dirshash ==NULL) {
+		dirshash = create_hash_table(FILE_RULE_TABLE_SIZE);
+		if (dirshash == NULL) {
+			yyerror("memory shortage\n");
+			exit(1);
+		}
+	}
+
+	domain -> dir_list = dirshash;
+	for (i=0; dir_list[i] != NULL; i++) {
+		s = insert_element(dirshash, "1", dir_list[i]);  
+		if(s == -2){
+			;
+		} else if(s < 0) {
+			bug_and_die("");
+		}
+		if(strcmp(dir_list[i],"~/") == 0) {
+			s = insert_element(dirshash, "1", dir_list[i]);  
+		}
+	}
+		  
 }
 
 /*for each domain, make dir_list hash table*/
-void make_dir_list(){
-  HASH_NODE **domain_array;
-  FILE_ACL_RULE *acl_array;
-  int i;
-  FILE_ACL_RULE acl;
-  int j;
-  char **dir_list;
-  DOMAIN *domain;
-
-  domain_array = create_hash_array(domain_hash_table);
-  for (i = 0; i < domain_hash_table->element_num; i++){
-    domain = (DOMAIN *)domain_array[i]->data;
-    if(domain->file_rule_array == NULL)
-      continue;
-    acl_array = domain->file_rule_array;
-    for(j = 0; j<domain->file_rule_array_num;j++){
-      acl = acl_array[j];
+ void make_dir_list() {
+	 HASH_NODE **domain_array;
+	 FILE_RULE *acl_array;
+	 int i;
+	 FILE_RULE acl;
+	 int j;
+	 char **dir_list;
+	 DOMAIN *domain;
+	 
+	 domain_array = create_hash_array(domain_hash_table);
+	 for (i = 0; i < domain_hash_table->element_num; i++) {
+		 domain = (DOMAIN *)domain_array[i]->data;
+		 if(domain->file_rule_array == NULL)
+			 continue;
+		 acl_array = domain->file_rule_array;
+		 for(j = 0; j<domain->file_rule_array_num; j++) {
+			 acl = acl_array[j];
      
-      if(acl.allowed!=DENY_PRM){
-	dir_list = get_dir_list(acl.path, converter_conf.homedir_list);
-	if(dir_list!=NULL){
-	  register_dirs(domain, dir_list);
-	}
-      }
-    }
-  }
-}
+			 if(acl.allowed!=DENY_PRM) {
+				 dir_list = get_dir_list(acl.path, converter_conf.homedir_list, 0);
+				 if (acl.state == FILE_DIR) {
+					 dir_list = extend_ntarray(dir_list, acl.path);    
+				 }
+
+				 if(dir_list != NULL) {
+					 register_dirs(domain, dir_list);
+				 }
+			 }
+		 }
+	 }
+ }
 
 char *get_disable_trans_boolean_name(char *domain){
   char *prefix;
@@ -1911,157 +1938,163 @@ void out_domain_trans_child_dir(FILE *, TRANS_RULE *, char *);
 
 /**
  *  @name:	out_domain_trans
- *  @about:	print domain_trans by using rulebuf
+ *  @about:	print domain_trans by using gDomain_trans_rules
  *  @args:	outfp (FILE *) -> output file descripter
  *  @return:	none
- *  @notes:	rulebuf is global variable
+ *  @notes:	gDomain_trans_rules is global variable
  */
-void out_domain_trans(FILE *outfp){
-  int i;
-  TRANS_RULE t;
-  FILE_LABEL *label;
-  char *name;/*domain name*/
-  int len;
-  int disable_trans_defined=0;
-  char *boolean_name=NULL;
-  /* print domain_auto_trans */
-  for (i = 0; i < domain_trans_rule_num; i++){
-    /* search target path's file label */
-    t = rulebuf[i];
-
-    disable_trans_defined = check_disable_trans_boolean_defined(t.child);
-    if(disable_trans_defined){
-
-      label = search_element(file_label_table, t.path);
-      if (label == NULL){
-	fprintf(stderr, "bug line %d\n", __LINE__);
-	exit(1);
-      }      
-      fprintf(outfp,"ifdef(`enable_mcs',`\n");
-
-      if(!ntarray_check_exist(converter_conf.mcs_range_trans_entry  ,t.path)){
-	fprintf(outfp,"range_transition unconfined_domain %s s0;\n",label->labelname);
-      }
-      fprintf(outfp,"')\n");    
-
-      boolean_name = get_disable_trans_boolean_name(t.child);
-      fprintf(outfp,"if(%s){}else{\n",boolean_name);
-      free(boolean_name);
-    }
-
-
-    name = strdup(t.parent);
-    len = strlen(name);	
-    /* convert "_r" to "_t" */
-    if(strcmp(name,"unconfined_domain")!=0){
-      if (len < 2){
-	fprintf(stderr, "bug line %d\n", __LINE__);
-      }
-      name[len-1] = 't';
-    }
+void out_domain_trans(FILE *outfp) {
+	int i;
+	TRANS_RULE t;
+	FILE_LABEL *label;
+	char *name;/*domain name*/
+	int len;
+	int disable_trans_defined=0;
+	char *boolean_name=NULL;
+	/* print domain_auto_trans */
+	for (i = 0; i < gDomain_trans_rule_num; i++) {
+		/* search target path's file label */
+		t = gDomain_trans_rules[i];		
+		disable_trans_defined = check_disable_trans_boolean_defined(t.child);
+		if(disable_trans_defined) {			
+			label = search_element(file_label_table, t.path);
+			if (label == NULL){
+				fprintf(stderr, "bug line %s %d\n", __FILE__, __LINE__);
+				exit(1);
+			}      
+			fprintf(outfp,"ifdef(`enable_mcs',`\n");
+			
+			if(!ntarray_check_exist(converter_conf.mcs_range_trans_entry  ,t.path)){
+				fprintf(outfp,"range_transition unconfined_domain %s s0;\n",label->labelname);
+			}
+			fprintf(outfp,"')\n");			
+			
+			boolean_name = get_disable_trans_boolean_name(t.child);
+			if(!gNoBool) {
+			fprintf(outfp,"if(%s){}else{\n",boolean_name);
+			}
+			free(boolean_name);
+		}
+		
+		name = strdup(t.parent);
+		len = strlen(name);	
+		/* convert "_r" to "_t" */
+		if(strcmp(name,"unconfined_domain")!=0){
+			if (len < 2){
+				fprintf(stderr, "bug line %d\n", __LINE__);
+			}
+			name[len-1] = 't';
+		}
        
-    if(strcmp(name,"unconfined_domain")==0){
-   
-      ;
-    }else if(search_element(domain_hash_table, name) == NULL){
-      fprintf(stderr, "Warning: domain %s does not exists for \"domain_trans %s %s\" rule. Skipped.\n", name, t.parent,t.path);
-      continue;
-    }else if(search_element(domain_hash_table, t.child) == NULL){
-      error_print(__FILE__, __LINE__, "bug! Aborted");
-      exit(1);
-    }
+		if(strcmp(name,"unconfined_domain")==0){
+			;
+		} else if (search_element(domain_hash_table, name) == NULL) {
+			fprintf(stderr, "Warning: domain %s does not exists for \"domain_trans %s %s\" rule. Skipped.\n", name, t.parent,t.path);
+			continue;
+		} else if (search_element(domain_hash_table, t.child) == NULL) {
+			error_print(__FILE__, __LINE__, "bug! Aborted");
+			exit(1);
+		}
+		
+		if(t.path == NULL) {
+			/*dynamic transition rule*/
+			fprintf(outfp, "domain_dyn_trans(%s,%s)\n", name, t.child);
+			continue;
+		}
+		
+		label = search_element(file_label_table, t.path);
 
-    if(t.path == NULL){
-      /*dynamic transition rule*/
-      fprintf(outfp, "domain_dyn_trans(%s,%s)\n", name, t.child);
-      continue;
-    }
-
-    label = search_element(file_label_table, t.path);
-    if (label == NULL){
-      fprintf(stderr, "bug line %d\n", __LINE__);
-      exit(1);
-    }
-    if (t.auto_flag == 1){
-      fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", name,label->labelname, t.child);
-      label = search_element(dir_label_table, t.path);
-      if(label)
-	fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", name,label->labelname, t.child);
-
-
-    } else {
-      fprintf(outfp, "domain_trans(%s,%s,%s)\n", name, label->labelname, t.child);
-      label = search_element(dir_label_table, t.path);
-      if(label)
-	fprintf(outfp, "domain_trans(%s,%s,%s)\n", name, label->labelname, t.child);
-    }
-    
-    
-    /*
-     * when entry point is directory(<dir>/ * or <dir>/ **) and a file under the directory is labeled,
-     * then output domain_auto_trans for label of the file.
-     */ 
-    out_domain_trans_child_dir(outfp, &t,name);
-
-    free(name);
-    if(disable_trans_defined){
-      /*close if(_disable_trans){}else{q*/
-      fprintf(outfp,"}\n");
-    }
-
-
-  }
+		if (label == NULL) {
+			bug_and_die("");
+		}
+		if(t.state == FILE_DIR) {
+			bug_and_die("");
+		} else {
+			if (t.auto_flag == 1) {
+				fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", name,label->labelname, t.child);
+				label = search_element(dir_label_table, t.path);
+				if(label)
+					fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", name,label->labelname, t.child);
+			} else {
+				fprintf(outfp, "domain_trans(%s,%s,%s)\n", name, label->labelname, t.child);
+				label = search_element(dir_label_table, t.path);
+				if(label)
+					fprintf(outfp, "domain_trans(%s,%s,%s)\n", name, label->labelname, t.child);
+			}
+		} 
+		if (t.state != FILE_FILE) {
+			/*
+			 * when entry point is directory(<dir>/ * or <dir>/ **) and a file under the directory is labeled,
+			 * then output domain_auto_trans for label of the file.
+			 */ 
+			out_domain_trans_child_dir(outfp, &t,name);
+		}
+		free(name);
+		if(disable_trans_defined) {
+			/*close if(_disable_trans){}else{q*/
+			if (!gNoBool)
+				fprintf(outfp,"}\n");
+		}
+	}
 }
 
 /**
  *  @name:	out_domain_trans_child_dir
- *  @about:	print domain_trans by using rulebuf
+ *  @about:	print domain_trans by using gDomain_trans_rules
  * 		attention:wildcard isn't supported,the program of the same name isn't supported.
  *  @args:	outfp (FILE *) -> output file descripter
  *  @return:	none
- *  @notes:	rulebuf is global variable
+ *  @notes:	gDomain_trans_rules is global variable
  */
-void out_domain_trans_child_dir(FILE *outfp, TRANS_RULE *t, char *domain_name){
+void out_domain_trans_child_dir(FILE *outfp, TRANS_RULE *t, char *domain_name) {
 	FILE_LABEL *label;
 	HASH_NODE **file_label_array;	
 	HASH_NODE **dir_label_array;
-
+	int s = FILE_FILE;
 	int i;
-	if(t->state ==  FILE_ITSELF){
-	  return;
+	if(t->state ==  FILE_DIR||FILE_FILE) {
+		return;
 	}
 	file_label_array = create_hash_array(file_label_table);
-
-	for (i = 0; i < file_label_table->element_num; i++){
-	  label = (FILE_LABEL *)file_label_array[i]->data;
-	  if(t->state == FILE_DIRECT_CHILD){
-	    if (chk_child_file(t->path, label->filename) == 1){
-	      fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
-	    }
-	  }
-	  if(t->state == FILE_ALL_CHILD){
-	    if(chk_child_dir(t->path,label->filename)==1){
-	      fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
-	    }
-	  }
+	
+	for (i = 0; i < file_label_table->element_num; i++) {
+		label = (FILE_LABEL *)file_label_array[i]->data;
+		if(t->state == FILE_DIRECT_CHILD) {
+			if (label->dir_flag) {
+				s = 0;
+			} else {
+				s = 1;
+			}
+			if (chk_child_file(t->path, label->filename, s) == 1){
+				fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
+			}
+		}
+		if(t->state == FILE_ALL_CHILD) {
+			if(chk_child_dir(t->path,label->filename) == 1) {
+				fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
+			}
+		}
 	}
 	dir_label_array = create_hash_array(dir_label_table);
 	
-	for (i = 0; i < dir_label_table->element_num; i++){
-	  label = (FILE_LABEL *)dir_label_array[i]->data;
-	  if(t->state == FILE_DIRECT_CHILD){
-	    if (chk_child_file(t->path, label->filename) == 1){
-	      fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
-	    }
-	  }
-	  if(t->state == FILE_ALL_CHILD){
-	    if(chk_child_dir(t->path,label->filename)==1){
-	      fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
-	    }
-	  }
-	}
-	
-
+	for (i = 0; i < dir_label_table->element_num; i++) {
+		label = (FILE_LABEL *)dir_label_array[i]->data;
+		if(t->state == FILE_DIRECT_CHILD) {	
+			if (label->dir_flag) {
+				s = 0;
+			} else {
+				s = 1;
+			}
+			if (chk_child_file(t->path, label->filename, s) == 1){
+				fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
+			}
+		}
+		if(t->state == FILE_ALL_CHILD) {
+			if(chk_child_dir(t->path,label->filename)==1){
+				fprintf(outfp, "domain_auto_trans(%s,%s,%s)\n", domain_name, label->labelname, t->child);
+			}
+		}
+	}	
 	free(file_label_array);
 }
 
