@@ -39,7 +39,7 @@
 #define CRONTAB_TYPE            "system_cron_spool_t"
 #define DUMMY_DOMAIN_NAME	"dummy_domain_t"
 #define DUMMY_ROLE_NAME	"dummy_role_r"
-#define FILE_ACL_TABLE_SIZE	1024
+#define FILE_RULE_TABLE_SIZE	1024
 
 #define INITRC_DIR              "/etc/rc.d/init.d"
 
@@ -61,18 +61,21 @@ typedef struct rbac_domain_t RBAC;
  *  this stores File Rule
  */
 /*To represent state */
-/*  allow <filename>*/
-#define FILE_ITSELF 0x01
+/*  allow <filename(not directory)>*/
+#define FILE_FILE 0x01
 /*allow <dir>*     */
 #define FILE_DIRECT_CHILD 0x02
 /*allow <dir>**    */
 #define FILE_ALL_CHILD 0x04
-typedef struct file_acl_rule_t{
+/* allow <directory> */
+#define FILE_DIR 0x08
+
+typedef struct file_rule_t{
   DOMAIN		*domain;	/* domain                */
   char		*path;		/* path name             */
   int		allowed;	/* toggle for permission */
-  int  state; /* FILE_DIRECT_CHILD   FILE_ALL_CHILD FILE_ITSELF */
-} FILE_ACL_RULE; 
+  int  state; /* FILE_DIRECT_CHILD   FILE_ALL_CHILD FILE_FILE FILE_DIR */
+} FILE_RULE; 
 
 
 /**
@@ -204,7 +207,7 @@ struct domain_t {
 
   
   /* File */
-  FILE_ACL_RULE *file_rule_array;
+  FILE_RULE *file_rule_array;
   int file_rule_array_num;
   HASH_TABLE	*appeared_file_name;   /*key:file name that appeared in allow/deny, value:1*/
 
@@ -266,10 +269,9 @@ typedef struct trans_rule_t
 {
 	char		*parent;
 	char		*path;
-  int state; /*state of path : FILE_DIRECT_CHILD   FILE_ALL_CHILD FILE_ITSELF */
+	int state; /*state of path : FILE_DIRECT_CHILD   FILE_ALL_CHILD FILE_DIR FILE_FILE */
 	char		*child;
 	int		auto_flag;	/* if the transition is "domain_auto_trans",then 1 */
-
 } TRANS_RULE;
 
 /**
@@ -279,7 +281,8 @@ typedef struct file_label_t
 {
 	char		*filename;
 	char		*labelname;
-	char		rec_flag;	/* if the label is inherited by child directory, this is 1,else 0 */
+	char		label_child_dir; /*If child dirs have to be labeled then 1 */
+	int             dir_flag;        /* if filename is directory this is 1*/
 } FILE_LABEL;
 
 
@@ -315,7 +318,7 @@ typedef struct user_role_t
 
 typedef struct entry_point_table_t{
   char *filename; /*wildcard is not permitted*/
-  int state; /*state of filename */ /* FILE_DIRECT_CHILD   FILE_ALL_CHILD FILE_ITSELF */
+  int state; /*state of filename */ /* FILE_DIRECT_CHILD   FILE_ALL_CHILD FILE_FILE FILE_DIR */
   char *to_domain;/*to domain*/
 } ENTRY_POINT;
 
@@ -354,8 +357,8 @@ extern int g_entry_point_array_num;
 /**
  *  domain transition
  */
-extern TRANS_RULE *rulebuf;			/* the body is in action.c			*/
-extern int domain_trans_rule_num;
+extern TRANS_RULE *gDomain_trans_rules;			/* the body is in action.c			*/
+extern int gDomain_trans_rule_num;
 
 /**
  *  relationship between user adn role
@@ -415,4 +418,29 @@ FILE *TMP_fp;
 
 #define MAX_COMMENT 256
 
+/*This indicates -t 1option*/
+extern int gDiet_by_attr;
+/*If macro is used more than that value, use attribute instead of attribute*/
+#define OPTIMIZE_THRESHOLD 3
+/*-p option*/
+extern int gProfile;
+
+/* whether dir search permission is supported or not */
+extern int gDir_search;
+
+/*--disable-boolean option*/
+extern int gNoBool;
+
+extern int  gOutFileTypeTransContext;
+
+extern  int gMoreWarning;
+
+extern char *gInFileTypeTransContext;
+
+
+#define bug_and_die(msg) { fprintf(stderr,"Bug in file %s in line %d, message:%s,exitting..\n",__FILE__, __LINE__, msg);exit(1);}
+
+/*dummy name, unprintable*/
+#define DUMMY_FILE_NAME "\xff" "childdir" "\xff"
+
 #endif
