@@ -23,10 +23,13 @@ import re
 import getopt
 import string
 import gettext
+from seedit.seedit import *
+
 
-gSeeditLoadConf="/usr/share/seedit/seedit-load.conf"
 gMakeFlags="CONFDIR=/etc/seedit/policy OUTDIR=/usr/share/seedit/sepolicy BASEPOLICYDIR=/usr/share/seedit/base_policy MACRODIR=/usr/share/seedit/macros"
 gAuditCtl="/sbin/auditctl"
+gStartStr ="# Added by seedit"
+gEndStr = "# End of seedit"
 
 def getConfinedDomains(filename):
     confinedDomains = []
@@ -71,12 +74,14 @@ def removeAuditChdirFromAuditRulesFile():
 
     lines=fh.readlines()
     lineBuf=[]
-    reg = re.compile("-a exit,always -S chdir")
+    sreg = re.compile(gStartStr)
+    flag = True
     for line in lines:
-        m = reg.search(line)
+        m = sreg.search(line)
         if m:
-            continue
-        lineBuf.append(line)
+            flag = False
+	if flag:
+        	lineBuf.append(line)
 
     fh.close()
 
@@ -125,6 +130,8 @@ def doAuditChdir():
     if confinedDomains==None:
         return
 
+    command = gAuditCtl+" -a exit,always -S chroot"
+    forkExec(command, gVerboseFlag)
     for domain in confinedDomains:
         command = gAuditCtl+" -a exit,always -S chdir -F obj_type="+domain
         command = command +" >/dev/null 2>&1 "
@@ -141,10 +148,12 @@ def doAuditChdir():
     lineBuf=[]
     for line in lines:
         lineBuf.append(line)
-    
+    lineBuf.append(gStartStr+"\n")
+    lineBuf.append("-a exit,always -S chroot\n") 
     for domain in confinedDomains:
         s = "-a exit,always -S chdir -F obj_type="+domain+"\n"
         lineBuf.append(s)
+    lineBuf.append(gEndStr+"\n")
 
     try:
         fh = open(filename,"w")
@@ -190,14 +199,15 @@ def doAuditChdirAll():
 
 
 def printUsage():
-    sys.stderr.write("seedit-load [-l(--load)] [-t(--test)] [-v(--verbose)] [-i (--init)]")
+    sys.stderr.write("seedit-load [-l(--load)] [-t(--test)] [-v(--verbose)] [-i (--init)]  [-d(--deploy)]")
     sys.stderr.write(_("\t-l\tDefault behavior.Load Symplified Policy to kernel, and restore label if labeling has been changed\n"))  
     sys.stderr.write(_("\t-t\tTest of seedit-converter\n"))
     sys.stderr.write(_("\t-i\tInitialize all file labels. This takes time.\n"))
     sys.stderr.write(_("\t-v\tVerbose output\n"))
     sys.stderr.write(_("\t-e\tVerbose output, to stderr\n"))
-    sys.stderr.write(_("\t-n\tDo not audit chdir logs(Effective only after FC5\n)"))
-    sys.stderr.write(_("\n-a\tAudit all chdir logs(Will generate lots of logs\n"))
+    sys.stderr.write(_("\t-n\tDo not audit chdir logs(Effective only after FC5)\n"))
+    sys.stderr.write(_("\t-a\tAudit all chdir logs(Will generate lots of logs\n"))
+    sys.stderr.write(_("\t-d\tDeploy poilcy to /etc/selinux/seedit. When cross install, it is installed in ./policy_root.\n"))
     sys.stderr.write(_("\t l,i,t option conflicts each other.\n"))
     sys.exit(1)
 
@@ -225,16 +235,34 @@ def doCommand(command):
 
     return 0
 def doLoad():
-    loadCommand = "cd /usr/share/seedit; make diffrelabel "+gMakeFlags+" 2>&1" 
+    if gCross:
+        loadCommand = "./seedit-cross.sh build 2>&1" 
+    else:
+        loadCommand = "cd /usr/share/seedit; make diffrelabel "+gMakeFlags+" 2>&1" 
+    return doCommand(loadCommand)
+
+def doDeploy():
+    if gCross:
+        loadCommand = "./seedit-cross.sh build 2>&1; ./seedit-cross.sh diff; ./seedit-cross.sh install 2>&1" 
+
+    else:
+        loadCommand = "cd /usr/share/seedit; make install "+gMakeFlags+" 2>&1" 
+
     return doCommand(loadCommand)
 
 def doInit():
+    if gCross:
+        return
+
     initCommand = "cd /usr/share/seedit; make relabel "+gMakeFlags+" 2>&1"
     print _("Initializing file labels it takes long time")
     return doCommand(initCommand)
 
 def doTest():
-    testCommand = "cd /usr/share/seedit; make policy "+gMakeFlags+"  2>&1" 
+    if gCross:
+        testCommand = "./seedit-cross.sh build  2>&1" 
+    else:
+        testCommand = "cd /usr/share/seedit; make policy "+gMakeFlags+"  2>&1" 
     return doCommand(testCommand)
 
 ####Main func
@@ -248,27 +276,28 @@ gAuditChdirFlag = True  #-n option
 gAuditChdirAllFlag = False #-a option
 
 # Check audit_chdir field in /etc/seedit/seedit-load.conf
-try:
-    input = open(gSeeditLoadConf, 'r')
+if gCross == False:
+    try:
+        input = open(gSeeditLoadConf, 'r')
     
-    lines = input.readlines()
-    for line in lines:
-        list = string.split(line)
-        try:
-            if list[0] == "audit_chdir":
-                if list[1] == "n":
-                    gAuditChdirFlag = False
-        except:
-            pass
+        lines = input.readlines()
+        for line in lines:
+            list = string.split(line)
+            try:
+                if list[0] == "audit_chdir":
+                    if list[1] == "n":
+                        gAuditChdirFlag = False
+            except:
+                pass
         input.close()
-except:
-    print "File Open Error:"+gSeeditLoadConf+" Skipped.\n"
+    except:
+        print "File Open Error:"+gSeeditLoadConf+" Skipped.\n"
 
-print "Audit chdir:"
-print gAuditChdirFlag
+    print "Audit chdir:"
+    print gAuditChdirFlag
 
 try:
-    opts, args = getopt.getopt(sys.argv[1:], "atnvei", ["audit","test","noaudit","verbose","init"])
+    opts, args = getopt.getopt(sys.argv[1:], "atnveird", ["audit","test","noaudit","verbose","init","remove-audit","deploy"])
 except getopt.GetoptError:
     printUsage()
 
@@ -295,6 +324,12 @@ for opt,arg in opts:
         if(gBehavior!=""):
             printUsage()
         gBehavior="load"
+    elif opt in ("-d", "--deploy"):
+        gBehavior="deploy"
+    elif opt in ("-r","--remove-audit"):
+        if(gBehavior!=""):
+            printUsage()
+        gBehavior="remove-audit"
 
 if gBehavior=="":
     gBehavior="load"
@@ -304,8 +339,12 @@ if os.path.exists("/usr/share/seedit/sepolicy/seedit-rbac-init"):
     print "Type /usr/share/seedit/script/seedit-installhelper.sh upgrade"
     sys.exit(1)
 
+if gBehavior == "remove-audit":
+    removeAuditChdir()
+    sys.exit(0)
+
 #Handles logging for chdir syscall
-if gBehavior != "test":
+if gBehavior not in  ("test","deploy") and gCross == False:
     removeAuditChdir()
     if gAuditChdirFlag==True:
         doAuditChdir()
@@ -320,6 +359,13 @@ elif gBehavior == "init":
     s= doInit()
 elif gBehavior =="test":
     s= doTest()
+elif gBehavior =="deploy":
+    s= doDeploy()
+    removeAuditChdir()
+    if gAuditChdirFlag == True and gCross == False:
+	doAuditChdir()
+    if gAuditChdirAllFlag == True and gCross == False:
+	doAuditChdirAll()
 
 if s<0:
     sys.exit(1)
